==> blam_1.8.4.orig.tar.gz: blam-1.8.4/src/ChannelList.cs <==
//
// Author: 
//   Mikael Hallendal <micke@imendio.com>
//
// (C) 2004 Imendio HB
// 

using Gdk;
using Gtk;
using GtkSharp;
using System;
using System.Collections;
using System.Runtime.InteropServices;
using Mono.Unix;

namespace Imendio.Blam {

    public class ChannelList : Gtk.TreeView {

        // Events
        public event ChannelEventHandler ChannelSelectedEvent;

        // Emitted when using right-click popup
        public event ChannelEventHandler RemoveChannelEvent;
        public event ChannelEventHandler MarkChannelAsReadEvent;
        public event ChannelEventHandler EditChannelEvent;
        public event ChannelEventHandler RefreshChannelEvent;

        private EventHandler selectionChangedHandler;

        private ChannelMenu popupMenu;

        private TreeViewColumn nameColumn;

        private IEnumerator channelEnumerator;

        public ChannelList(IList channels)
        { 
            TreeViewColumn col;
            CellRenderer   cell;

            /* Channel name column */
            nameColumn = new TreeViewColumn();
            cell = new CellRendererText();

            nameColumn.PackStart(cell, true);
            nameColumn.AddAttribute (cell, "text", 1);
            nameColumn.AddAttribute (cell, "weight", 2);

            AppendColumn(nameColumn);

            /* Items column */
            col = new TreeViewColumn();
            cell = new CellRendererText();

            col.PackStart(cell, true);
            col.SetCellDataFunc(cell,
                                new TreeCellDataFunc(ItemsCellDataFunc));

            AppendColumn(col);

            this.RulesHint = true;

            selectionChangedHandler = new EventHandler(SelectionChanged);
            this.Selection.Changed += selectionChangedHandler;

            this.Model = new ListStore (typeof(Channel), 
                                        typeof(string), typeof (int));
            this.HeadersVisible = false;

            // Sort the list
            mChannelSortFunc = new SortFunc (ChannelSortFunc);
            Utils.TreeSetSortFunc (this.Model.Handle, 1, 
                                   mChannelSortFunc);
            ((ListStore)this.Model).SetSortColumnId (1, SortType.Ascending);    

            // Right click popup
            this.popupMenu = new ChannelMenu();
            this.popupMenu.EditSelected       += EditChannelCb;
            this.popupMenu.MarkAsReadSelected += MarkAsReadCb;
            this.popupMenu.RemoveSelected     += RemoveChannelCb;
            this.popupMenu.RefreshSelected    += RefreshChannelCb;

            channelEnumerator = channels.GetEnumerator();

            GLib.Idle.Add(new GLib.IdleHandler(IdleAdd));
        }

        private bool IdleAdd()
        {
            while (channelEnumerator.MoveNext ()) {
                Channel channel = (Channel)channelEnumerator.Current;
                Add(channel);
            }

            return false;
        }

        public void Add (Channel channel)
        {
            int weight = (int)Pango.Weight.Normal;

            if (channel.NrOfUnreadItems > 0) {
                weight = (int)Pango.Weight.Bold;
            }

            ((ListStore)this.Model).AppendValues(channel, channel.Name, weight);
        }

        public Channel GetSelected ()
        {
            TreeModel model;
            TreeIter  iter;

            if (!this.Selection.GetSelected(out model, out iter)) {
                return null;
            }

            return(Channel)model.GetValue(iter, 0);
        }

        public void Updated (Channel channel)
        {
            TreeIter iter = FindChannel (channel);

            if (!iter.Equals(TreeIter.Zero)) {
                int weight = (int)Pango.Weight.Normal;

                if (channel.NrOfUnreadItems > 0) {
                    weight = (int)Pango.Weight.Bold;
                }

                string oldName = (string) this.Model.GetValue (iter, 1);
                if (oldName != channel.Name) {
                    this.Model.SetValue (iter, 1, channel.Name);
                }
                this.Model.SetValue (iter, 2, weight);

                this.Model.EmitRowChanged (this.Model.GetPath(iter), iter);
            }
        }

        public void Remove (Channel channel)
        {
            TreeIter iter = FindChannel(channel);
            bool valid;

            if (!iter.Equals(TreeIter.Zero)) {
                this.Selection.Changed -= selectionChangedHandler;

                valid = ((ListStore)this.Model).Remove(ref iter);

                this.Selection.Changed += selectionChangedHandler;

                if (!valid) {
                    EmitChannelSelected(null);
                    return;
                }

                this.Selection.SelectIter(iter);
            }
        }

        public bool NextUnread ()
        {
            TreeModel model;
            TreeIter  iter;

            if (this.Selection.GetSelected(out model, out iter)) {
                Channel channel = (Channel)model.GetValue(iter, 0);
                if (channel.NrOfUnreadItems > 0) {
                    return true;
                }

                if (!this.Model.IterNext(ref iter)) {
                    if (!this.Model.GetIterFirst(out iter)) {
                        return false;
                    }
                }
            } else {
                if (!this.Model.GetIterFirst(out iter)) {
                    return false;
                }
            }

            TreeIter startIter = iter;

            do {
                Channel channel = (Channel)model.GetValue(iter, 0);
                if (channel.NrOfUnreadItems > 0) {
                    this.Selection.SelectIter(iter);
                    ScrollToCell(this.Model.GetPath(iter), nameColumn,
                                 false, 0, 0);
                    return true;
                }

                if (!this.Model.IterNext(ref iter)) {
                    this.Model.GetIterFirst(out iter);
                }
            } while (!iter.Equals(startIter));

            return false;
        }

        private void EditChannelCb()
        {
            EmitEditChannelEvent(GetSelected ());
        }

        private void MarkAsReadCb ()
        {
            if (MarkChannelAsReadEvent != null) {
                MarkChannelAsReadEvent (GetSelected ());
            }
        }

        private void RemoveChannelCb ()
        {
            if (RemoveChannelEvent != null) {
                RemoveChannelEvent (GetSelected ());
            }
        }

        private void RefreshChannelCb ()
        {
            if (RefreshChannelEvent != null) {
                RefreshChannelEvent (GetSelected ());
            }
        }

        private SortFunc mChannelSortFunc;
        private int ChannelSortFunc (IntPtr a, IntPtr b)
        {
            TreeIter iterA = TreeIter.New (a);
            TreeIter iterB = TreeIter.New (b);

            Channel channelA = (Channel)this.Model.GetValue (iterA, 0);
            Channel channelB = (Channel)this.Model.GetValue (iterB, 0);

            return channelA.Name.CompareTo (channelB.Name);
        }

        protected override bool OnButtonPressEvent (EventButton bEvent)
        {
            switch (bEvent.Button) {
            case 1:
                if (bEvent.Type == EventType.TwoButtonPress) {
                    EditChannelCb();
                } else {
                    return base.OnButtonPressEvent (bEvent);
                }
                break;
            case 3:
                TreePath path;

                if (!GetPathAtPos ((int) bEvent.X, (int) bEvent.Y, out path)) {
                    return false;
                }

                this.Selection.SelectPath (path);

                popupMenu.Activate (bEvent);

//		popupMenu.Popup((uint)bEvent.XRoot, (uint)bEvent.YRoot, 
//				bEvent.Time);
                return false;
            }

            return false;
        }

        private void SelectionChanged(object obj, EventArgs args)
        {
            TreeSelection selection = (TreeSelection)obj;
            TreeIter      iter;
            TreeModel     model;
            Channel       channel;

            if (!selection.GetSelected(out model, out iter)) {
                EmitChannelSelected(null);
                return;
            }

            channel = (Channel)model.GetValue(iter, 0);
            if (channel != null) {
                EmitChannelSelected(channel);
            }
        }

        private void ItemsCellDataFunc(TreeViewColumn col,
                                       CellRenderer   cell,
                                       TreeModel      model,
                                       TreeIter       iter)
        {
            Channel channel = (Channel)model.GetValue(iter, 0);
            int weight = (int)Pango.Weight.Normal;

            ((CellRendererText)cell).Text = channel.NrOfUnreadItems + "/" + channel.NrOfItems;
            if (channel.NrOfUnreadItems > 0) {
                weight = (int)Pango.Weight.Bold;
            }

            ((CellRendererText)cell).Weight = weight;
        }

        private void EmitChannelSelected(Channel channel)
        {
            if (ChannelSelectedEvent != null) {
                ChannelSelectedEvent(channel);
            }
        }

        // Used by ChannelUpdated
        private Channel findChannel;
        private TreeIter foundIter;
        private bool ForeachFindChannel(TreeModel model, 
                                        TreePath  path,
                                        TreeIter  iter)
        {
            Channel channel = (Channel)model.GetValue(iter, 0);
            if (channel == findChannel) {
                foundIter = iter;
                return true;
            }

            return false;
        }

        private TreeIter FindChannel(Channel channel) 
        {
            findChannel = channel;
            foundIter = TreeIter.Zero;

            this.Model.Foreach(new TreeModelForeachFunc(ForeachFindChannel));

            return foundIter;
        }

        private void EmitEditChannelEvent(Channel channel)
        {
            if (EditChannelEvent != null) {
                EditChannelEvent(channel);
            }
        }
    }

    class ChannelMenu {
        public delegate void MenuItemSelectedHandler();

        public event MenuItemSelectedHandler EditSelected;
        public event MenuItemSelectedHandler MarkAsReadSelected;
        public event MenuItemSelectedHandler RemoveSelected;
        public event MenuItemSelectedHandler RefreshSelected;

        public void Activate (Gdk.EventButton eb) 
        {
            Gtk.Menu popup_menu = new Gtk.Menu ();

            GtkUtil.AppendMenuItem (popup_menu, 
                                    Catalog.GetString ("_Mark as read"),
                                    new EventHandler (EmitMarkAsReadCb));
            GtkUtil.AppendMenuItem (popup_menu, 
                                    Catalog.GetString ("_Refresh"),
                                    "gtk-refresh",
                                    new EventHandler (EmitRefreshChannelCb),
                                    true);
            GtkUtil.AppendMenuSeparator (popup_menu);
            GtkUtil.AppendMenuItem (popup_menu,
                                    Catalog.GetString ("_Edit"),
                                    new EventHandler (EmitEditCb));
            GtkUtil.AppendMenuItem (popup_menu,
                                    Catalog.GetString ("Remo_ve"),
                                    "gtk-remove",
                                    new EventHandler (EmitRemoveCb), true);
            popup_menu.Popup (null, null, null, 
                              eb.Button, eb.Time);
        }

        public void EmitMarkAsReadCb (object obj, EventArgs args)
        {
            if (MarkAsReadSelected != null) {
                MarkAsReadSelected ();
            }
        }

        public void EmitRefreshChannelCb (object obj, EventArgs args)
        {
            if (RefreshSelected != null) {
                RefreshSelected();
            }
        }

        public void EmitEditCb (object obj, EventArgs args) 
        {
            if (EditSelected != null) {
                EditSelected();
            }
        }

        public void EmitRemoveCb (object obj, EventArgs args)
        {
            if (RemoveSelected != null) {
                RemoveSelected();
            }
        }
    }
}

==> blam_1.8.6.orig.tar.gz: blam-1.8.6/src/ChannelList.cs <==
//
// Author: 
//   Mikael Hallendal <micke@imendio.com>
//
// (C) 2004 Imendio HB
// 

using Gdk;
using Gtk;
using GtkSharp;
using System;
using System.Collections;
using System.Runtime.InteropServices;
using Mono.Unix;

namespace Imendio.Blam {

    public class ChannelList : Gtk.TreeView {

        // Events
        public event ChannelEventHandler ChannelSelectedEvent;

        // Emitted when using right-click popup
        public event ChannelEventHandler RemoveChannelEvent;
        public event ChannelEventHandler MarkChannelAsReadEvent;
        public event ChannelEventHandler EditChannelEvent;
        public event ChannelEventHandler RefreshChannelEvent;

        private EventHandler selectionChangedHandler;

        private ChannelMenu popupMenu;

        private TreeViewColumn nameColumn;

        private IEnumerator channelEnumerator;

        public ChannelList(IList channels)
        { 
            TreeViewColumn col;
            CellRenderer   cell;

            /* Channel name column */
            nameColumn = new TreeViewColumn();
            cell = new CellRendererText();

            nameColumn.PackStart(cell, true);
            nameColumn.Sizing = TreeViewColumnSizing.Autosize;
            nameColumn.Expand = true;
            nameColumn.SetCellDataFunc(cell, new TreeCellDataFunc(NamesCellDataFunc));

            AppendColumn(nameColumn);

            /* Items column */
            col = new TreeViewColumn();
            cell = new CellRendererText();

            col.PackStart(cell, true);
            col.SetCellDataFunc(cell,
                                new TreeCellDataFunc(ItemsCellDataFunc));

            AppendColumn(col);

            this.RulesHint = true;

            selectionChangedHandler = new EventHandler(SelectionChanged);
            this.Selection.Changed += selectionChangedHandler;

            this.Model = new ListStore (typeof(Channel));
            this.HeadersVisible = false;

            // Sort the list
            (Model as ListStore).DefaultSortFunc = ChannelSort;
            ((ListStore)this.Model).SetSortColumnId (-1, SortType.Ascending);

            // Right click popup
            this.popupMenu = new ChannelMenu();
            this.popupMenu.EditSelected       += EditChannelCb;
            this.popupMenu.MarkAsReadSelected += MarkAsReadCb;
            this.popupMenu.RemoveSelected     += RemoveChannelCb;
            this.popupMenu.RefreshSelected    += RefreshChannelCb;

            channelEnumerator = channels.GetEnumerator();

            GLib.Idle.Add(new GLib.IdleHandler(IdleAdd));
        }

        private void ForceResort()
        {
            (Model as ListStore).SetSortColumnId(-1, SortType.Descending);
            (Model as ListStore).SetSortColumnId(-1, SortType.Ascending);
        }

        private bool IdleAdd()
        {
            while (channelEnumerator.MoveNext ()) {
                Channel channel = (Channel)channelEnumerator.Current;
                Add(channel);
            }

            return false;
        }

        public void Add (Channel channel)
        {
            ((ListStore)this.Model).AppendValues(channel);
        }

        public Channel GetSelected ()
        {
            TreeModel model;
            TreeIter  iter;

            if (!this.Selection.GetSelected(out model, out iter)) {
                return null;
            }

            return(Channel)model.GetValue(iter, 0);
        }

        public void Updated (Channel channel)
        {
            TreeIter iter = FindChannel (channel);

            if (!iter.Equals(TreeIter.Zero)) {
                this.Model.EmitRowChanged (this.Model.GetPath(iter), iter);
            }

            ForceResort();

        }

        public void Remove (Channel channel)
        {
            TreeIter iter = FindChannel(channel);
            bool valid;

            if (!iter.Equals(TreeIter.Zero)) {
                this.Selection.Changed -= selectionChangedHandler;

                valid = ((ListStore)this.Model).Remove(ref iter);

                this.Selection.Changed += selectionChangedHandler;

                if (!valid) {
                    EmitChannelSelected(null);
                    return;
                }

                this.Selection.SelectIter(iter);
            }
        }

        public bool NextUnread ()
        {
            TreeModel model;
            TreeIter  iter;

            if (this.Selection.GetSelected(out model, out iter)) {
                Channel channel = (Channel)model.GetValue(iter, 0);
                if (channel.NrOfUnreadItems > 0) {
                    return true;
                }

                if (!this.Model.IterNext(ref iter)) {
                    if (!this.Model.GetIterFirst(out iter)) {
                        return false;
                    }
                }
            } else {
                if (!this.Model.GetIterFirst(out iter)) {
                    return false;
                }
            }

            TreeIter startIter = iter;

            do {
                Channel channel = (Channel)model.GetValue(iter, 0);
                if (channel.NrOfUnreadItems > 0) {
                    this.Selection.SelectIter(iter);
                    ScrollToCell(this.Model.GetPath(iter), nameColumn,
                                 false, 0, 0);
                    return true;
                }

                if (!this.Model.IterNext(ref iter)) {
                    this.Model.GetIterFirst(out iter);
                }
            } while (!iter.Equals(startIter));

            return false;
        }

        private void EditChannelCb()
        {
            EmitEditChannelEvent(GetSelected ());
        }

        private void MarkAsReadCb ()
        {
            if (MarkChannelAsReadEvent != null) {
                MarkChannelAsReadEvent (GetSelected ());
            }
        }

        private void RemoveChannelCb ()
        {
            if (RemoveChannelEvent != null) {
                RemoveChannelEvent (GetSelected ());
            }
        }

        private void RefreshChannelCb ()
        {
            if (RefreshChannelEvent != null) {
                RefreshChannelEvent (GetSelected ());
            }
        }

        private int ChannelSort(TreeModel model, TreeIter ia, TreeIter ib)
        {
            Channel a = Model.GetValue(ia, 0) as Channel;
            Channel b = Model.GetValue(ib, 0) as Channel;

            return a.Name.CompareTo(b.Name);
        }

        protected override bool OnButtonPressEvent (EventButton bEvent)
        {
            switch (bEvent.Button) {
            case 1:
                if (bEvent.Type == EventType.TwoButtonPress) {
                    EditChannelCb();
                } else {
                    return base.OnButtonPressEvent (bEvent);
                }
                break;
            case 3:
                TreePath path;

                if (!GetPathAtPos ((int) bEvent.X, (int) bEvent.Y, out path)) {
                    return false;
                }

                this.Selection.SelectPath (path);

                popupMenu.Activate (bEvent);

//		popupMenu.Popup((uint)bEvent.XRoot, (uint)bEvent.YRoot, 
//				bEvent.Time);
                return false;
            }

            return false;
        }

        private void SelectionChanged(object obj, EventArgs args)
        {
            TreeSelection selection = (TreeSelection)obj;
            TreeIter      iter;
            TreeModel     model;
            Channel       channel;

            if (!selection.GetSelected(out model, out iter)) {
                EmitChannelSelected(null);
                return;
            }

            channel = (Channel)model.GetValue(iter, 0);
            if (channel != null) {
                EmitChannelSelected(channel);
            }
        }

        private void NamesCellDataFunc(TreeViewColumn col, CellRenderer cell, TreeModel model, TreeIter iter)
        {
            Channel channel = model.GetValue(iter, 0) as Channel;
            int weight = (int)Pango.Weight.Normal;

            (cell as CellRendererText).Text = channel.Name;
            if(channel.NrOfUnreadItems > 0){
                weight = (int)Pango.Weight.Bold;
            }

            (cell as CellRendererText).Ellipsize = Pango.EllipsizeMode.End;
            (cell as CellRendererText).Weight = weight;
        }

        private void ItemsCellDataFunc(TreeViewColumn col,
                                       CellRenderer   cell,
                                       TreeModel      model,
                                       TreeIter       iter)
        {
            Channel channel = (Channel)model.GetValue(iter, 0);
            int weight = (int)Pango.Weight.Normal;

            ((CellRendererText)cell).Text = channel.NrOfUnreadItems + "/" + channel.NrOfItems;
            if (channel.NrOfUnreadItems > 0) {
                weight = (int)Pango.Weight.Bold;
            }

            ((CellRendererText)cell).Xalign = 1.0f;
            ((CellRendererText)cell).Weight = weight;
        }

        private void EmitChannelSelected(Channel channel)
        {
            if (ChannelSelectedEvent != null) {
                ChannelSelectedEvent(channel);
            }
        }

        // Used by ChannelUpdated
        private Channel findChannel;
        private TreeIter foundIter;
        private bool ForeachFindChannel(TreeModel model, 
                                        TreePath  path,
                                        TreeIter  iter)
        {
            Channel channel = (Channel)model.GetValue(iter, 0);
            if (channel == findChannel) {
                foundIter = iter;
                return true;
            }

            return false;
        }

        private TreeIter FindChannel(Channel channel) 
        {
            findChannel = channel;
            foundIter = TreeIter.Zero;

            this.Model.Foreach(new TreeModelForeachFunc(ForeachFindChannel));

            return foundIter;
        }

        private void EmitEditChannelEvent(Channel channel)
        {
            if (EditChannelEvent != null) {
                EditChannelEvent(channel);
            }
        }
    }

    class ChannelMenu {
        public delegate void MenuItemSelectedHandler();

        public event MenuItemSelectedHandler EditSelected;
        public event MenuItemSelectedHandler MarkAsReadSelected;
        public event MenuItemSelectedHandler RemoveSelected;
        public event MenuItemSelectedHandler RefreshSelected;

        public void Activate (Gdk.EventButton eb) 
        {
            Gtk.Menu popup_menu = new Gtk.Menu ();

            GtkUtil.AppendMenuItem (popup_menu, 
                                    Catalog.GetString ("_Mark as read"),
                                    new EventHandler (EmitMarkAsReadCb));
            GtkUtil.AppendMenuItem (popup_menu, 
                                    Catalog.GetString ("_Refresh"),
                                    "gtk-refresh",
                                    new EventHandler (EmitRefreshChannelCb),
                                    true);
            GtkUtil.AppendMenuSeparator (popup_menu);
            GtkUtil.AppendMenuItem (popup_menu,
                                    Catalog.GetString ("_Edit"),
                                    "gtk-edit",
                                    new EventHandler (EmitEditCb),
                                    true);
            GtkUtil.AppendMenuItem (popup_menu,
                                    Catalog.GetString ("Remo_ve"),
                                    "gtk-remove",
                                    new EventHandler (EmitRemoveCb), true);
            popup_menu.Popup (null, null, null, 
                              eb.Button, eb.Time);
        }

        public void EmitMarkAsReadCb (object obj, EventArgs args)
        {
            if (MarkAsReadSelected != null) {
                MarkAsReadSelected ();
            }
        }

        public void EmitRefreshChannelCb (object obj, EventArgs args)
        {
            if (RefreshSelected != null) {
                RefreshSelected();
            }
        }

        public void EmitEditCb (object obj, EventArgs args) 
        {
            if (EditSelected != null) {
                EditSelected();
            }
        }

        public void EmitRemoveCb (object obj, EventArgs args)
        {
            if (RemoveSelected != null) {
                RemoveSelected();
            }
        }
    }
}
